/*****************************************************************************/
/*                                                                           */
/*  MODULE         RXASYNC.cmd                                               */
/*                                                                           */
/*  VERSION        Version 1.5 - 23rd May 1994                               */
/*                                                                           */
/*  COPYRIGHT      Copyright (c) 1993, 1994 by Crucial Applications          */
/*                              All rights reserved.                         */
/*                                                                           */
/*  AUTHOR         Ian Timms                                                 */
/*                                                                           */
/*  DESCRIPTION                                                              */
/*                                                                           */
/*    Test the essential RXASYNC.DLL functions to make sure they work.       */
/*                                                                           */
/*    NOTE: Only those functions indicated with '+' are tested by this       */
/*          routine.                                                         */
/*                                                                           */
/*  FUNCTIONS                                                                */
/*                                                                           */
/*       House keeping                                                       */
/*                                                                           */
/*  + RxAsyncLoadFuncs     -   Load the functions into REXX                  */
/*  + RxAsyncDropFuncs     -   Drop the functions from REXX                  */
/*                                                                           */
/*       Process control                                                     */
/*                                                                           */
/*  + RxAsyncPriority      -   Set Rexx process priority                     */
/*  + RxAsyncSleep         -   Wait for some defined period                  */
/*                                                                           */
/*       ASYNC IOCTL interface                                               */
/*                                                                           */
/*  + RxAsyncGetDcbInfo    -   Get the device control block info             */
/*  + RxAsyncSetDcbInfo    -   Set the device control block info             */
/*  + RxAsyncGetEnhParm    -   Get the enhanced parameters                   */
/*  + RxAsyncSetEnhParm    -   Set the enhanced parameters                   */
/*  + RxAsyncGetLnCtrl     -   Get the comms port settings                   */
/*  + RxAsyncSetLnCtrl     -   Set the comms port settings                   */
/*    RxAsyncSetBreakOff   -   Turn break off                                */
/*    RxAsyncSetBreakOn    -   Turn break on                                 */
/*    RxAsyncTxImmediate   -   Transmit byte immediately                     */
/*    RxAsyncTxStop        -   Stop transmission (ala XOFF)                  */
/*    RxAsyncTxStart       -   Start transmission (ala XON)                  */
/*    RxAsyncGetComStatus  -   Get COM status                                */
/*    RxAsyncGetTxStatus   -   Get Transmit status                           */
/*    RxAsyncGetMdmOutput  -   Get modem output signals                      */
/*  + RxAsyncSetMdmOutput  -   Set modem output signals                      */
/*    RxAsyncGetMdmInput   -   Get modem input signals                       */
/*    RxAsyncGetRxCount    -   Get size and number of chars in RX queue      */
/*    RxAsyncGetTxCount    -   Get size and number of chars in TX queue      */
/*    RxAsyncGetComError   -   Get the COM Error Word                        */
/*    RxAsyncGetComEvent   -   Get the COM Event Word                        */
/*                                                                           */
/*       Extensions                                                          */
/*                                                                           */
/*  + RxAsyncOpen          -   Open the specified comms port                 */
/*  + RxAsyncClose         -   Close the specified comms port                */
/*  + RxAsyncWait          -   Wait for receipt of input                     */
/*  + RxAsyncRead          -   Read data and wait if necessary               */
/*  + RxAsyncWrite         -   Write data to the comms port                  */
/*  + RxAsyncPrint         -   Print string on stdout                        */
/*    RxAsyncCarrier       -   Test for Data Carrier Detect (DCD)            */
/*  + RxAsyncBreak         -   Send break for defined period                 */
/*  + RxAsyncDcbInfo       -   Print the device control block info           */
/*                                                                           */
/*       File handling                                                       */
/*                                                                           */
/*  + RxAsyncFileExists    -   Test for file(s) existence                    */
/*  + RxAsyncFileDelete    -   Delete specified file                         */
/*  + RxAsyncFileCopy      -   Copy specified file                           */
/*  + RxAsyncFileMove      -   Move specified file                           */
/*  + RxAsyncFileRename    -   Rename specified file                         */
/*  + RxAsyncFileNameIs    -   Determine file name                           */
/*  + RxAsyncFilePathIs    -   Determine file path                           */
/*  + RxAsyncDirectory     -   Query/Change directory/drive                  */
/*                                                                           */
/*****************************************************************************/

call on error

/*--------------------------------------------------------------*/
/* NOTE: Adjust the following as required by your configuration */
/*--------------------------------------------------------------*/

PortName      = 'COM4'
newBaud       = 19200
newData       = 8
newParity     = 'N'
newStop       = 1
newWriteLim   = 50
newReadLim    = 50
newFlags1     = '00001001'
newFlags2     = '10100000'
newFlags3     = '11010010'
newErrChar    = '00'
newBrkChar    = '00'
newXonChar    = '11'
newXoffChar   = '13'
newEnhParms   = '00000010'

/*--------------------------------------------------------------*/

PortHandle    = ''
oldBaud       = 0
oldData       = 0
oldParity     = ''
oldStop       = 0
oldWriteLim   = 0
oldReadLim    = 0
oldFlags1     = ''
oldFlags2     = ''
oldFlags3     = ''
oldErrChar    = ''
oldBrkChar    = ''
oldXonChar    = ''
oldXoffChar   = ''
oldEnhParms   = ''

True          = 1
False         = 0
Regular       = 2
Critical      = 3
NormalLvl     = 0
InpStr        = ''
Remaining     = 0
crlf          = D2C(13)''D2C(10)
none_on       = '00'
dtr_on        = '01'
rts_on        = '02'
both_on       = '03'
none_off      = 'FF'
dtr_off       = 'FE'
rts_off       = 'FD'
both_off      = 'FC'

/*--------------------------------------------------------------*/

/*-----------------------------*/
/* Setup                       */
/*-----------------------------*/

/* load functions */

call RxFuncAdd 'RxAsyncLoadFuncs', 'RXASYNC', 'RxAsyncLoadFuncs'
call RxAsyncLoadFuncs
Say 'RxAsyncLoadFuncs is complete'

/* set session priority */

rc = RxAsyncPriority( Critical, NormalLvl )
if rc <> 0
   then say 'RxAsyncPriority failed with rc=>'rc'<'
   else say 'RxAsyncPriority set, Class=>'Critical'<, Level=>'NormalLvl'<'

/* open device */

rc = RxAsyncOpen( PortName, 'PortHandle' )
if rc <> 0
   then say 'RxAsyncOpen failed with rc=>'rc'<'
   else say 'RxAsyncOpen opened the device handle=>'PortHandle'<'

/*-----------------------*/
/* Line Control Handling */
/*-----------------------*/

/* save original line control settings */

rc = RxAsyncGetLnCtrl( PortHandle, 'oldBaud', 'oldData', 'oldParity', 'oldStop' )
if rc <> 0
   then say 'RxAsyncGetLnCtrl failed with rc=>'rc'<'
   else say 'RxAsyncGetLnCtrl saved the original settings'

say 'Baudrate >'oldBaud'<'
say 'Databits >'oldData'<'
say 'Parity   >'oldParity'<'
say 'Stopbits >'oldStop'<'

/* set new line control settings */

rc = RxAsyncSetLnCtrl( PortHandle, newBaud, newData, newParity, newStop )
if rc <> 0
   then say 'RxAsyncSetLnCtrl failed with rc=>'rc'<'
   else say 'RxAsyncSetLnCtrl set the new settings'

/* query new line control settings */

rc = RxAsyncGetLnCtrl( PortHandle, 'newBaud', 'newData', 'newParity', 'newStop' )
if rc <> 0
   then say 'RxAsyncGetLnCtrl failed with rc=>'rc'<'
   else say 'RxAsyncGetLnCtrl queried the new settings'

say 'Baudrate >'newBaud'<'
say 'Databits >'newData'<'
say 'Parity   >'newParity'<'
say 'Stopbits >'newStop'<'

/*-------------------------------*/
/* Device Control Block Handling */
/*-------------------------------*/

/* save original dcb info settings */

rc = RxAsyncGetDcbInfo( PortHandle, 'oldWriteLim', 'oldReadLim', 'oldFlags1', 'oldFlags2', 'oldFlags3', 'oldErrChar', 'oldBrkChar', 'oldXonChar', 'oldXoffChar' )
if rc <> 0
   then say 'RxAsyncGetDcbInfo failed with rc=>'rc'<'
   else say 'RxAsyncGetDcbInfo saved the original settings'

oldFlags1 = X2B(oldFlags1)
oldFlags2 = X2B(oldFlags2)
oldFlags3 = X2B(oldFlags3)

say 'Write Timout   >'oldWriteLim'<'
say 'Read  Timout   >'oldReadLim'<'
say 'F1 HandShake   >'oldFlags1'<'
say 'F2 FlowReplace >'oldFlags2'<'
say 'F3 Timeout     >'oldFlags3'<'
say 'Error Replace  >'oldErrChar'<'
say 'Break Replace  >'oldBrkChar'<'
say 'Xon character  >'oldXonChar'<'
say 'Xoff character >'oldXoffChar'<'

/* set new dcb info settings */

rc = RxAsyncSetDcbInfo( PortHandle, newWriteLim, newReadLim, newFlags1, newFlags2, newFlags3, newErrChar, newBrkChar, newXonChar, newXoffChar )
if rc <> 0
   then say 'RxAsyncSetDcbInfo failed with rc=>'rc'<'
   else say 'RxAsyncSetDcbInfo set the new settings'

/* query new dcb info settings */

rc = RxAsyncGetDcbInfo( PortHandle, 'newWriteLim', 'newReadLim', 'newFlags1', 'newFlags2', 'newFlags3', 'newErrChar', 'newBrkChar', 'newXonChar', 'newXoffChar' )
if rc <> 0
   then say 'RxAsyncGetDcbInfo failed with rc=>'rc'<'
   else say 'RxAsyncGetDcbInfo queried the new settings'

newFlags1 = X2B(newFlags1)
newFlags2 = X2B(newFlags2)
newFlags3 = X2B(newFlags3)

say 'Write Timout   >'newWriteLim'<'
say 'Read  Timout   >'newReadLim'<'
say 'F1 HandShake   >'newFlags1'<'
say 'F2 FlowReplace >'newFlags2'<'
say 'F3 Timeout     >'newFlags3'<'
say 'Error Replace  >'newErrChar'<'
say 'Break Replace  >'newBrkChar'<'
say 'Xon character  >'newXonChar'<'
say 'Xoff character >'newXoffChar'<'

/*-----------------------------*/
/* Enhanced Parameter Handling */
/*-----------------------------*/

/* save original enhanced parms settings */

rc = RxAsyncGetEnhParm( PortHandle, 'oldEnhParms' )
if rc <> 0
   then say 'RxAsyncGetEnhParm failed with rc=>'rc'<'
   else say 'RxAsyncGetEnhParm saved the original settings'

oldEnhParms = X2B(oldEnhParms)

say 'Enhanced Parms >'oldEnhParms'<'

/* set new enhanced parms settings */

rc = RxAsyncSetEnhParm( PortHandle, newEnhParms )
if rc <> 0
   then say 'RxAsyncSetEnhParm failed with rc=>'rc'<'
   else say 'RxAsyncSetEnhParm set the new settings'

/* query new enhanced parms settings */

rc = RxAsyncGetEnhParm( PortHandle, 'newEnhParms' )
if rc <> 0
   then say 'RxAsyncGetEnhParm failed with rc=>'rc'<'
   else say 'RxAsyncGetEnhParm queried the new settings'

newEnhParms = X2B(newEnhParms)

say 'Enhanced Parms >'newEnhParms'<'

/*----------------------*/
/* Read, Write and Wait */
/*----------------------*/

/* write modem reset string */

rc = RxAsyncWrite( PortHandle, 0, 'ATZ'||D2C(13), 'Remaining' )
if rc <> 0
   then say 'Modem not turned on, >'Remaining'< chars unwritten'
   else say 'RxAsyncWrite output ATZ<cr>'

/* read modem response */

InpStr = ""
GetOk = True
do while GetOk
   rc = RxAsyncRead( PortHandle, 0, 3000, 'InpStr' )
   GetOk = False
   select
     when rc <> 0                 then say 'RxAsyncRead failed with rc=>'rc'<'
     when InpStr == "OK"||crlf    then say 'RxAsyncRead returned >'InpStr'<'
     when InpStr == "ERROR"||crlf then say 'RxAsyncRead returned >'InpStr'<'
     otherwise
        say 'RxAsyncRead returned >'InpStr'<'
        GetOk = True
   end
end

/* write AT<cr> string */

rc = RxAsyncWrite( PortHandle, 0, 'AT'D2C(13), 'Remaining' )
if rc <> 0
   then say 'Modem not turned on, >'Remaining'< chars unwritten'
   else say 'RxAsyncWrite output AT<cr>'

/* read modem response */

InpStr = ""
GetOk = True
do while GetOk
   rc = RxAsyncRead( PortHandle, 0, 3000, 'InpStr' )
   GetOk = False
   select
     when rc <> 0                 then say 'RxAsyncRead failed with rc=>'rc'<'
     when InpStr == "OK"||crlf    then say 'RxAsyncRead returned >'InpStr'<'
     when InpStr == "ERROR"||crlf then say 'RxAsyncRead returned >'InpStr'<'
     otherwise
        say 'RxAsyncRead returned >'InpStr'<'
        GetOk = True
   end
end

/* write bad string */

rc = RxAsyncWrite( PortHandle, 0, 'AT CAUSE AN ERROR'||D2C(13), 'Remaining' )
if rc <> 0
   then say 'Modem not turned on, >'Remaining'< chars unwritten'
   else say 'RxAsyncWrite output AT CAUSE AN ERROR<cr>'

/* read modem response */

InpStr = ""
GetOk = True
do while GetOk
   rc = RxAsyncRead( PortHandle, 0, 3000, 'InpStr' )
   GetOk = False
   select
     when rc <> 0                 then say 'RxAsyncRead failed with rc=>'rc'<'
     when InpStr == "OK"||crlf    then say 'RxAsyncRead returned >'InpStr'<'
     when InpStr == "ERROR"||crlf then say 'RxAsyncRead returned >'InpStr'<'
     otherwise
        say 'RxAsyncRead returned >'InpStr'<'
        GetOk = True
   end
end

/* write a query string */

rc = RxAsyncWrite( PortHandle, 0, 'ATS7?'||D2C(13), 'Remaining' )
if rc <> 0
   then say 'Modem not turned on, >'Remaining'< chars unwritten'
   else say 'RxAsyncWrite output ATS7?<cr>'

/* read modem response */

InpStr = ""
GetOk = True
do while GetOk
   rc = RxAsyncRead( PortHandle, 0, 3000, 'InpStr' )
   GetOk = False
   select
     when rc <> 0                 then say 'RxAsyncRead failed with rc=>'rc'<'
     when InpStr == "OK"||crlf    then say 'RxAsyncRead returned >'InpStr'<'
     when InpStr == "ERROR"||crlf then say 'RxAsyncRead returned >'InpStr'<'
     otherwise
        say 'RxAsyncRead returned >'InpStr'<'
        GetOk = True
   end
end

/* write modem attention string */

rc = RxAsyncWrite( PortHandle, 0, 'AT'D2C(13), 'Remaining' )
if rc <> 0
   then say 'Modem may not be turned on, >'Remaining'< chars unwritten'
   else say 'RxAsyncWrite output AT<cr>'

/* wait three seconds or until there is data to be read */

say 'Wait for data 'time()
rc = RxAsyncWait( PortHandle, 3000 )
say 'Waited enough 'time()
if rc = 0
   then say 'RxAsyncWait detected input'
   else if rc = -1
           then say 'RxAsyncWait timed out'
           else say 'RxAsyncWait returned rc=>'rc'<'

/* read modem response */

InpStr = ""
GetOk = True
do while GetOk
   rc = RxAsyncRead( PortHandle, 0, 3000, 'InpStr' )
   GetOk = False
   select
     when rc <> 0                 then say 'RxAsyncRead failed with rc=>'rc'<'
     when InpStr == "OK"||crlf    then say 'RxAsyncRead returned >'InpStr'<'
     when InpStr == "ERROR"||crlf then say 'RxAsyncRead returned >'InpStr'<'
     otherwise
        say 'RxAsyncRead returned >'InpStr'<'
        GetOk = True
   end
end

/* write query string */

rc = RxAsyncWrite( PortHandle, 0, 'ATS1?'||D2C(13), 'Remaining' )
if rc <> 0
   then say 'Modem not turned on, >'Remaining'< chars unwritten'
   else say 'RxAsyncWrite output ATS1?<cr>'

/* read modem response */

InpStr = ""
GetOk = True
do while GetOk
   rc = RxAsyncRead( PortHandle, 0, 3000, 'InpStr' )
   GetOk = False
   select
     when rc <> 0                 then say 'RxAsyncRead failed with rc=>'rc'<'
     when InpStr == "OK"||crlf    then say 'RxAsyncRead returned >'InpStr'<'
     when InpStr == "ERROR"||crlf then say 'RxAsyncRead returned >'InpStr'<'
     otherwise
        say 'RxAsyncRead returned >'InpStr'<'
        GetOk = True
   end
end

/*---------------*/
/* Miscellaneous */
/*---------------*/

/* send break for half a second */

BreakTime = 500
rc = RxAsyncBreak( PortHandle, BreakTime )
if rc <> 0
   then say 'RxAsyncBreak failed with rc=>'rc'<'
   else say 'RxAsyncBreak sent break for 'BreakTime' milliseconds'

/* print dcb info */

rc = RxAsyncDcbInfo( PortHandle )
if rc <> 0
   then say 'RxAsyncDcbInfo failed with rc=>'rc'<'
   else say 'RxAsyncDcbInfo returned okay'

/* print something on stdout */

call RxAsyncPrint crlf||'Well, how do you like it so far.<g>'||crlf

/* turn DTR off (DTR is normally on once port is open) */

rc = RxAsyncSetMdmOutput( PortHandle, none_on, dtr_off )
if rc <> 0
   then say 'RxAsyncSetMdmOutput failed with rc=>'rc'<'
   else say 'RxAsyncSetMdmOutput disabled DTR'

/* sleep 2 seconds for modem to recognise DTR toggle */

call RxAsyncSleep 2000

/* turn DTR on */

rc = RxAsyncSetMdmOutput( PortHandle, dtr_on, none_off )
if rc <> 0
   then say 'RxAsyncSetMdmOutput failed with rc=>'rc'<'
   else say 'RxAsyncSetMdmOutput enabled DTR'

/*-----------------------------*/
/* File and Directory Handling */
/*-----------------------------*/

fdir = RxAsyncDirectory('.')
fnam = fdir||'RXASYNC'

if RxAsyncFileExists( fnam'.cmd' )
   then say 'File 'fnam'.cmd was found'
   else say 'File 'fnam'.cmd was not found'

if RxAsyncFileCopy( fnam'.cmd', fnam'.xxx' )
   then say 'File 'fnam'.cmd copied to 'fnam'.xxx'
   else say 'File 'fnam'.cmd not copied to 'fnam'.xxx'

if RxAsyncFileMove( fnam'.xxx', fnam'.yyy' )
   then say 'File 'fnam'.xxx was moved to 'fnam'.yyy'
   else say 'File 'fnam'.xxx was not moved to 'fnam'.yyy'

if RxAsyncFileRename( fnam'.yyy', fnam'.xxx' )
   then say 'File 'fnam'.yyy was renamed to 'fnam'.xxx'
   else say 'File 'fnam'.yyy was not renamed to 'fnam'.xxx'

if RxAsyncFileDelete( fnam'.xxx' )
   then say 'File 'fnam'.xxx was deleted'
   else say 'File 'fnam'.xxx was not deleted'

say 'Path is 'RxAsyncFilePathIs( fnam'.cmd' )

say 'Name is 'RxAsyncFileNameIs( fnam'.cmd' )

/*-----------------------------*/
/* Cleanup                     */
/*-----------------------------*/

/* reset to original enhanced parms settings */

rc = RxAsyncSetEnhParm( PortHandle, oldEnhParms )
if rc <> 0
   then say 'RxAsyncSetEnhParm failed with rc=>'rc'<'
   else say 'RxAsyncSetEnhParm restored the original settings'

/* reset to original dcb info settings */

rc = RxAsyncSetDcbInfo( PortHandle, oldWriteLim, oldReadLim, oldFlags1, oldFlags2, oldFlags3, oldErrChar, oldBrkChar, oldXonChar, oldXoffChar )
if rc <> 0
   then say 'RxAsyncSetDcbInfo failed with rc=>'rc'<'
   else say 'RxAsyncSetDcbInfo restored the original settings'

/* reset to original line control settings */

rc = RxAsyncSetLnCtrl( PortHandle, oldBaud, oldData, oldParity, oldStop )
if rc <> 0
   then say 'RxAsyncSetLnCtrl failed with rc=>'rc'<'
   else say 'RxAsyncSetLnCtrl restored the original settings'

/* close the device */

rc = RxAsyncClose( PortHandle )
if rc <> 0
   then say 'RxAsyncClose failed with rc=>'rc'<'
   else say 'RxAsyncClose closed the device'

/* reset session priority */

rc = RxAsyncPriority( Regular, NormalLvl )
if rc <> 0
   then say 'RxAsyncPriority failed with rc=>'rc'<'
   else say 'RxAsyncPriority set, Class=>'Regular'<, Level=>'NormalLvl'<'

/* drop functions */

call RxAsyncDropFuncs
Say 'RxAsyncDropFuncs is complete'

exit

/* just in case<g> */

error:
   call RxAsyncClose PortHandle
   exit

